/**
 * Copyright (c) 2020 Bubble, Inc.  All rights reserved.
 * For personal (non-commercial) use, see license: https://getbubblenow.com/bubble-license/
 */
package bubble.notify;

import bubble.dao.account.AccountDAO;
import bubble.dao.bill.AccountPlanDAO;
import bubble.dao.cloud.BubbleNodeDAO;
import bubble.model.account.Account;
import bubble.model.bill.AccountPlan;
import bubble.model.cloud.AnsibleInstallType;
import bubble.model.cloud.notify.ReceivedNotification;
import bubble.service.account.SyncAccountNotification;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;

import static org.cobbzilla.util.json.JsonUtil.json;
import static org.cobbzilla.wizard.server.RestServerBase.reportError;

@Slf4j
public class NotificationHandler_sync_account extends ReceivedNotificationHandlerBase {

    @Autowired private BubbleNodeDAO nodeDAO;
    @Autowired private AccountDAO accountDAO;
    @Autowired private AccountPlanDAO accountPlanDAO;

    @Override
    public void handleNotification(ReceivedNotification n) {
        final var node = nodeDAO.findByUuid(n.getFromNode());
        if (node == null) {
            log.warn("sync_account: node not found: " + n.getFromNode());
            return;
        }

        final var notification = json(n.getPayloadJson(), SyncAccountNotification.class);

        final var localAccount = accountDAO.findByUuid(notification.getAccountUuid());
        if (localAccount == null) {
            reportError("sync_account: localAccount not found: " + notification.getAccountUuid());
            return;
        }
        if (!localAccount.sync()) {
            log.info("sync_account: localAccount " + localAccount.getName() + " has sync disabled, no sync done");
            return;
        }

        if (notification.getHashedPassword().isPresent()) {
            syncAccountPassword(localAccount, notification.getHashedPassword().get());
        }
        if (notification.getAccountPlan().isPresent()) {
            syncAccountPlan(localAccount.getUuid(), notification.getAccountPlan().get());
        }
        if (notification.getForceDeletedAccountPlanName().isPresent()) {
            syncForceDelAccountPlan(localAccount.getUuid(), notification.getForceDeletedAccountPlanName().get());
        }
    }

    private void syncAccountPassword(@NonNull final Account localAccount,
                                     @NonNull final String incomingHashedPassword) {
        localAccount.getHashedPassword().setHashedPassword(incomingHashedPassword);

        // if we are a node, set skipSync so we don't get caught in an infinite loop
        // (the node would notify the sage, which would notify the node, ad infinitum)
        localAccount.setSkipSync(configuration.getThisNetwork().getInstallType() == AnsibleInstallType.node);

        // update password, if we are a sage, this will notify all networks of password change
        accountDAO.update(localAccount);
    }

    private void syncAccountPlan(@NonNull final String accountUuid, @NonNull final AccountPlan incomingPlan) {
        final var localPlan = accountPlanDAO.findByAccountAndId(accountUuid, incomingPlan.getName());

        if (localPlan == null) {
            // create new localPlan
            final var newPlan = new AccountPlan(incomingPlan);
            newPlan.setSkipSync(configuration.getThisNetwork().getInstallType() == AnsibleInstallType.node);
            accountPlanDAO.create(newPlan);
        } else {
            // update localPlan
            localPlan.update(incomingPlan);
            localPlan.setSkipSync(configuration.getThisNetwork().getInstallType() == AnsibleInstallType.node);
            accountPlanDAO.update(localPlan);
        }
    }

    private void syncForceDelAccountPlan(@NonNull final String accountUuid, @NonNull final String planName) {
        final var localPlan = accountPlanDAO.findByAccountAndId(accountUuid, planName);

        if (configuration.getThisNetwork().getInstallType() == AnsibleInstallType.node) {
            accountPlanDAO.forceDeleteWithoutSync(localPlan.getUuid());
        } else {
            accountPlanDAO.forceDelete(localPlan.getUuid());
        }
    }

}
